@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 # -*- coding: UTF-8 -*-
 
-#   Copyright (C) 2021  Andrew Bauer
+#   Copyright (C) 2022  Andrew Bauer
 #   Copyright (C) 2014  Enno Rodegerdts
 
 #   This program is free software; you can redistribute it and/or modify
@@ -33,16 +33,41 @@ from skyalmanac.lib import increments as increments
 from skyalmanac.lib.alma_skyfield import init_sf as init_sf
 
 
-def makePDF(fn, msg = ""):
-    command = 'pdflatex {}'.format(fn + ".tex")
-    os.system(command)
-    print("finished" + msg)
+def deletePDF(filename):
+    if os.path.exists(filename + ".pdf"):
+        try:
+            os.remove(filename + ".pdf")
+        except PermissionError:
+            print("ERROR: please close '{}' so it can be re-created".format(filename + ".pdf"))
+            sys.exit(0)
+    if os.path.exists(filename + ".tex"):
+        os.remove(filename + ".tex")
+
+def makePDF(args, fn, msg = ""):
+    command = 'pdflatex {}'.format(args + fn + ".tex")
+    if args == "":
+        os.system(command)
+        print("finished" + msg)
+    else:
+        returned_value = os.system(command)
+        if returned_value != 0:
+            if msg != "":
+                print("ERROR detected while" + msg)
+            else:
+                print("!!   ERROR detected while creating PDF file   !!")
+                print("!! Append '-v' or '-log' for more information !!")
+        else:
+            if msg != "":
+                print("finished" + msg)
+            else:
+                print("finished creating PDF")
     return
 
 def tidy_up(fn):
-    os.remove(fn + ".tex")
-    if os.path.isfile(fn + ".log"):
-        os.remove(fn + ".log")
+    if not keeptex: os.remove(fn + ".tex")
+    if not keeplog:
+        if os.path.isfile(fn + ".log"):
+            os.remove(fn + ".log")
     if os.path.isfile(fn + ".aux"):
         os.remove(fn + ".aux")
     return
@@ -52,10 +77,28 @@ def tidy_up(fn):
 if sys.version_info[0] < 3:
     raise Exception("Must be using Python 3")
 
-if config.ephndx not in set([0, 1, 2]):
+if config.ephndx not in set([0, 1, 2, 3, 4]):
     print("Error - Please choose a valid ephemeris in config.py")
     sys.exit(0)
 
+try:
+    arg = sys.argv[1]
+except IndexError:
+    arg = ""
+if len(sys.argv) > 2 or not (arg == "" or arg == "-v" or arg == "-log" or arg == "-tex"):
+    print("One optional command line parameter is permitted:")
+    print(" python -m skyalmanac -v")
+    print(" ... to send pdfTeX output to the terminal")
+    print(" python -m skyalmanac -log")
+    print(" ... to keep the log file")
+    print(" python -m skyalmanac -tex")
+    print(" ... to keep the tex file")
+    sys.exit(0)
+
+args = "" if arg == "-v" else "-interaction=batchmode -halt-on-error "
+keeplog = True if arg == "-log" else False
+keeptex = True if arg == "-tex" else False
+
 d = datetime.datetime.utcnow().date()
 first_day = datetime.date(d.year, d.month, d.day)
 
@@ -156,13 +199,11 @@ if s in set(['1', '2', '3', '4', '5']):
         t   Traditional
         m   Modern
 """)
-        ff = '_'
         DecFmt = ''
         config.tbls = tsin[0:1]	# table style
         config.decf = tsin[1:2]	# Declination format
         if config.tbls != 'm':
             config.tbls = ''		# anything other than 'm' is traditional
-            ff = ''
         if config.decf != '+':		# Positive/Negative Declinations
             config.decf = ''		# USNO format for Declination
         else:
@@ -178,7 +219,9 @@ if s in set(['1', '2', '3', '4', '5']):
             print(msg)
 ##            config.writeLOG(msg)
             first_day = datetime.date(yearint, 1, 1)
-            fn = "almanac{}{}".format(ff,year+DecFmt)
+            ff = "NAtrad_" if config.tbls != 'm' else "NAmod_"
+            fn = "{}{}".format(ff,year+DecFmt)
+            deletePDF(f_prefix + fn)
             outfile = open(fn + ".tex", mode="w", encoding="utf8")
             outfile.write(tables.almanac(first_day,122,df180,df360,dfcm))
             outfile.close()
@@ -187,7 +230,7 @@ if s in set(['1', '2', '3', '4', '5']):
             print(msg)
 ##            config.writeLOG("\n\n" + msg + "\n")
             print()
-            makePDF(fn, " creating nautical almanac for {}".format(year))
+            makePDF(args, fn, " creating nautical almanac for {}".format(year))
             tidy_up(fn)
 ##        config.closeLOG()     # close log after the for-loop
 
@@ -197,11 +240,12 @@ if s in set(['1', '2', '3', '4', '5']):
             msg = "\nCreating the sun tables for the year {}\n".format(year)
             print(msg)
             first_day = datetime.date(yearint, 1, 1)
-            fn = "sunalmanac{}{}".format(ff,year+DecFmt)
+            ff = "STtrad_" if config.tbls != 'm' else "STmod_"
+            fn = "{}{}".format(ff,year+DecFmt)
             outfile = open(fn + ".tex", mode="w", encoding="utf8")
             outfile.write(suntables.almanac(first_day,25,dfra))
             outfile.close()
-            makePDF(fn, " creating sun tables for {}".format(year))
+            makePDF(args, fn, " creating sun tables for {}".format(year))
             tidy_up(fn)
 
     elif s == '3':      # Nautical almanac   -  6 days from today
@@ -209,7 +253,9 @@ if s in set(['1', '2', '3', '4', '5']):
         start = time.time()
         msg = "\nCreating nautical almanac tables - from {}\n".format(sdmy)
         print(msg)
-        fn = "almanac{}{}".format(ff,symd+DecFmt)
+        ff = "NAtrad_" if config.tbls != 'm' else "NAmod_"
+        fn = "{}{}".format(ff,symd+DecFmt)
+        deletePDF(f_prefix + fn)
         outfile = open(fn + ".tex", mode="w", encoding="utf8")
         outfile.write(tables.almanac(first_day,2,df180,df360,dfcm))
         outfile.close()
@@ -219,27 +265,30 @@ if s in set(['1', '2', '3', '4', '5']):
 ##        config.writeLOG('\n\n' + msg)
 ##        config.closeLOG()
         print()
-        makePDF(fn)
+        makePDF(args, fn)
         tidy_up(fn)
 
     elif s == '4':      # Sun tables only    - 30 days from today
         msg = "\nCreating the sun tables - from {}\n".format(sdmy)
         print(msg)
-        fn = "sunalmanac{}{}".format(ff,symd+DecFmt)
+        ff = "STtrad_" if config.tbls != 'm' else "STmod_"
+        fn = "{}{}".format(ff,symd+DecFmt)
+        deletePDF(f_prefix + fn)
         outfile = open(fn + ".tex", mode="w", encoding="utf8")
         outfile.write(suntables.almanac(first_day,2,dfra))
         outfile.close()
-        makePDF(fn)
+        makePDF(args, fn)
         tidy_up(fn)
 
     elif s == '5':
         msg = "\nCreating the Increments and Corrections tables\n"
         print(msg)
         fn = "inc"
+        deletePDF(f_prefix + fn)
         outfile = open(fn + ".tex", mode="w", encoding="utf8")
         outfile.write(increments.makelatex())
         outfile.close()
-        makePDF(fn)
+        makePDF(args, fn)
         tidy_up(fn)
 
 else:
